mod cpp_s;
mod ts_c;

#[derive(Debug, Clone, clap::ValueEnum)]
pub enum ServerGenerators {
    Cpp
}

#[derive(Debug, Clone, clap::ValueEnum)]
pub enum ClientGenerators {
    Ts
}

impl ServerGenerators {
    pub fn generate(&self, file_base_name: &std::path::PathBuf, rpc: &crate::data::RPC) {
        match self {
            Self::Cpp => cpp_s::gen(file_base_name, rpc)
        }
    }
}

impl ClientGenerators {
    pub fn generate(&self, file_base_name: &std::path::PathBuf, rpc: &crate::data::RPC) {
        match self {
            Self::Ts => ts_c::gen(file_base_name, rpc)
        }
    }
}

pub struct IndentedWriter {
    pub ident: usize,
    pub f: std::fs::File,
    indent_next: bool
}

impl IndentedWriter {
    pub fn new(f: std::fs::File) -> Self {
        Self {
            ident: 0,
            f,
            indent_next: false
        }
    }
}

impl std::io::Write for IndentedWriter {
    fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
        for b in buf {
            if b == &b'}' && self.ident > 0 {
                self.ident -= 1;
            }

            if b == &b'\n' {
                self.indent_next = true;
            } else if self.indent_next {
                self.indent_next = false;
                for _ in 0..self.ident {
                    self.f.write_all(b"    ")?;
                }
            }

            if b == &b'{' {
                self.ident += 1;
            }

            self.f.write_all(&[*b])?;
        }
        Ok(buf.len())
    }

    fn flush(&mut self) -> std::io::Result<()> {
        self.f.flush()
    }
}

